/******************************************************************************
 * @file     main.c
 * @brief	 NUC123 series USBD driver source file
 * @version	 2.0.0
 * @date	 22, March, 2013
 *
 * @note
 * Copyright (C) 2013 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include <stdio.h>
#include "NUC123Series.h"
#include "cdc_serial.h"


#define PLL_CLOCK           72000000

/*--------------------------------------------------------------------------*/
STR_VCOM_LINE_CODING gLineCoding0 = {115200, 0, 0, 8};  /* Baud rate : 115200    */
                                                        /* Stop bit     */
                                                        /* parity       */
                                                        /* data bits    */
STR_VCOM_LINE_CODING gLineCoding1 = {115200, 0, 0, 8};  /* Baud rate : 115200    */
                                                        /* Stop bit     */
                                                        /* parity       */
                                                        /* data bits    */

uint16_t gCtrlSignal0 = 0;     /* BIT0: DTR(Data Terminal Ready) , BIT1: RTS(Request To Send) */
uint16_t gCtrlSignal1 = 0;     /* BIT0: DTR(Data Terminal Ready) , BIT1: RTS(Request To Send) */

/*--------------------------------------------------------------------------*/
#define RXBUFSIZE           512 /* RX buffer size */  
#define TXBUFSIZE           512 /* RX buffer size */  

#define TX_FIFO_SIZE_0       16  /* TX Hardware FIFO size */
#define TX_FIFO_SIZE_1       16  /* TX Hardware FIFO size */


/*---------------------------------------------------------------------------------------------------------*/
/* Global variables                                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
/* UART0 */
volatile uint8_t comRbuf0[RXBUFSIZE];
volatile uint16_t comRbytes0 = 0;
volatile uint16_t comRhead0 = 0;
volatile uint16_t comRtail0 = 0;

volatile uint8_t comTbuf0[TXBUFSIZE];
volatile uint16_t comTbytes0 = 0;
volatile uint16_t comThead0 = 0;
volatile uint16_t comTtail0 = 0;

uint8_t gRxBuf0[64] = {0};
uint8_t *gpu8RxBuf0 = 0;
uint32_t gu32RxSize0 = 0;
uint32_t gu32TxSize0 = 0;

volatile int8_t gi8BulkOutReady0 = 0;

/* UART1 */
volatile uint8_t comRbuf1[RXBUFSIZE];
volatile uint16_t comRbytes1 = 0;
volatile uint16_t comRhead1 = 0;
volatile uint16_t comRtail1 = 0;

volatile uint8_t comTbuf1[TXBUFSIZE];
volatile uint16_t comTbytes1 = 0;
volatile uint16_t comThead1 = 0;
volatile uint16_t comTtail1 = 0;

uint8_t gRxBuf1[64] = {0};
uint8_t *gpu8RxBuf1 = 0;
uint32_t gu32RxSize1 = 0;
uint32_t gu32TxSize1 = 0;

volatile int8_t gi8BulkOutReady1 = 0;


/*--------------------------------------------------------------------------*/
void SYS_Init(void)
{
    UNLOCKREG();						//en: Unlock the locked registers
    
    GPIO_XT1();						//en: Enable XTAL PINs
    CLK_HXT_ENABLE();				//en: Enable XTAL
    while(!CLK_IS_STABLE(CLK_HXT_STB));	//en: Wait XTAL Stable
    
    //en: configure PLL
    SET_PLL_CLKSRC(CLK_PLLSRC_HXT);		//en: Configure PLL Source is HXT
	/* PLL = 144MHz	*/
	CLK_SET_PLL_NO(CLK_PLLCON_NO_1);
	CLK_SET_PLL_NR(3);
	CLK_SET_PLL_NF(36);  					// 12 *	36/3 * 1/1 = 144MHz
    CLK_PLL_PD_DISABLE();
    CLK_PLL_OUTP_EN();
    while(!CLK_IS_STABLE(CLK_PLL_STB));		//en: Wait PLL Stable

	/* When PLL = 144MHz, USB = PLL/3 = 48MHz, HCLK = PLL/2 = 72MHz	*/
	SET_USB_CLKDIV(3);
	SET_HCLK_CLKDIV(2);

	/* Switch HCLK to PLL output 144/3 MHz */
	SET_HCLK_CLKSRC(CLK_HCLK_PLL);			// Switch system clock HCLK source to PLL output
   
	 /* Enable USB device clock */
    APBCLK_ENABLE(CLK_USBD);

    SystemCoreClockUpdate();				//en: Update system core clock

    LOCKREG(); //en: Re-lock the locked registers
}


void UART0_Init(void)
{
    UNLOCKREG();						//en: Unlock the locked registers

	GPIO_UART0_RXTX_PC();     //Only set UART RX/TX pin.PC4=RX,PC5=TX
    SYS_RESET_IP2(SYS_RST_UART0);                             //En:Reset UART0 IP
    APBCLK_ENABLE(CLK_UART0);                             //En:Enable UART0 clock
    SET_UART_CLKSRC(CLK_UART_HIRC);                        //En:Select external crystal as UART clock source
    //SET_UART_CLKDIV(2);                                      //En:Set UART clock divider number
    UART_SET_BAUD_MODE2(UART0, __IRC22M, 115200);               //En:Set UART0 Baudrate
    UART_SET_DATA_FORMAT(UART0, UART_WORD_LEN_8 | UART_PARITY_NONE | UART_STOP_BIT_1); //En:Configure UART0 data format
    UART_ENABLE_INT(UART0, UART_IER_RDA_IEN|UART_IER_RLS_IEN);
    NVIC_EnableIRQ(UART0_IRQn);

    LOCKREG(); //en: Re-lock the locked registers
}


void UART1_Init(void)
{
    UNLOCKREG();						//en: Unlock the locked registers

    GPIO_UART1_RXTX();
    SYS_RESET_IP2(SYS_RST_UART1);                             //En:Reset UART0 IP
    APBCLK_ENABLE(CLK_UART1);                             //En:Enable UART0 clock
    SET_UART_CLKSRC(CLK_UART_HIRC);                        //En:Select external crystal as UART clock source
    //SET_UART_CLKDIV(2);                                      //En:Set UART clock divider number
    UART_SET_BAUD_MODE2(UART1, __IRC22M, 115200);               //En:Set UART0 Baudrate
    UART_SET_DATA_FORMAT(UART1, UART_WORD_LEN_8 | UART_PARITY_NONE | UART_STOP_BIT_1); //En:Configure UART0 data format
    UART_ENABLE_INT(UART1, UART_IER_RDA_IEN|UART_IER_RLS_IEN);
    NVIC_EnableIRQ(UART1_IRQn);

    LOCKREG(); //en: Re-lock the locked registers
}
/*---------------------------------------------------------------------------------------------------------*/
/* UART Callback function                                                                           	   */
/*---------------------------------------------------------------------------------------------------------*/
void UART0_IRQHandler(void)
{
    uint32_t u32IntStatus;
	uint8_t bInChar;
	int32_t size;

    u32IntStatus = UART0->ISR;
    
	if((u32IntStatus & 0x1 /* RDAIF */) || (u32IntStatus & 0x10 /* TOUT_IF */))
	{
        /* Receiver FIFO threashold level is reached or Rx time out */

		/* Get all the input characters */
		while((UART0->FSR & UART_FSR_RX_EMPTY_Msk) == 0)
        {
			/* Get the character from UART Buffer */
            bInChar = UART0->DATA;

			/* Check if buffer full */
			if(comRbytes0 < RXBUFSIZE)
			{
				/* Enqueue the character */
				comRbuf0[comRtail0++] = bInChar;
				if(comRtail0 >= RXBUFSIZE)
				    comRtail0 = 0;
				comRbytes0++;
			}
			else
			{
			    /* FIFO over run */
			}			
		}
	}
	
	if(u32IntStatus & 0x2 /* THRE_IF */)
	{   
				   
		if(comTbytes0)
		{
			/* Fill the Tx FIFO */
			size = comTbytes0;
			if(size >= TX_FIFO_SIZE_0)
			{
				size = TX_FIFO_SIZE_0;   
			}
			
			while(size)
			{
				bInChar = comTbuf0[comThead0++];
				UART0->DATA = bInChar;
                if(comThead0 >= TXBUFSIZE)
				    comThead0 = 0;
				comTbytes0--;
				size--;
			}
		}
		else
		{
		    /* No more data, just stop Tx (Stop work) */
            UART0->IER &= (~UART_IER_THRE_IEN_Msk);
		}
	}
	
}


void UART1_IRQHandler(void)
{
    uint32_t u32IntStatus;
	uint8_t bInChar;
	int32_t size;

    u32IntStatus = UART1->ISR;
    
	if((u32IntStatus & 0x1 /* RDAIF */) || (u32IntStatus & 0x10 /* TOUT_IF */))
	{
        /* Receiver FIFO threashold level is reached or Rx time out */

		/* Get all the input characters */
		while((UART1->FSR & UART_FSR_RX_EMPTY_Msk) == 0)
        {
			/* Get the character from UART Buffer */
            bInChar = UART1->DATA;

			/* Check if buffer full */
			if(comRbytes1 < RXBUFSIZE)
			{
				/* Enqueue the character */
				comRbuf1[comRtail1++] = bInChar;
				if(comRtail1 >= RXBUFSIZE)
				    comRtail1 = 0;
				comRbytes1++;
			}
			else
			{
			    /* FIFO over run */
			}			
		}
	}
	
	if(u32IntStatus & 0x2 /* THRE_IF */)
	{   
				   
		if(comTbytes1)
		{
			/* Fill the Tx FIFO */
			size = comTbytes1;
			if(size >= TX_FIFO_SIZE_1)
			{
				size = TX_FIFO_SIZE_1;   
			}
			
			while(size)
			{
				bInChar = comTbuf1[comThead1++];
				UART1->DATA = bInChar;
                if(comThead1 >= TXBUFSIZE)
				    comThead1 = 0;
				comTbytes1--;
				size--;
			}
		}
		else
		{
		    /* No more data, just stop Tx (Stop work) */
            UART1->IER &= (~UART_IER_THRE_IEN_Msk);
		}
	}
}

void VCOM_TransferData(void)
{
	int32_t i, i32Len;

    /* Check if any data to send to USB & USB is ready to send them out */
    if(comRbytes0 && (gu32TxSize0 == 0))
    {

        i32Len = comRbytes0;
        if(i32Len > EP2_MAX_PKT_SIZE)
            i32Len = EP2_MAX_PKT_SIZE;
    
        for(i=0;i<i32Len;i++)
        {
            gRxBuf0[i] = comRbuf0[comRhead0++];
            if(comRhead0 >= RXBUFSIZE)
                comRhead0 = 0;
        }
        
        NVIC_DisableIRQ(UART0_IRQn);
        comRbytes0 -= i32Len;
        NVIC_EnableIRQ(UART0_IRQn);
        
        gu32TxSize0 = i32Len;
        USBD_MemCopy((uint8_t *)(USBD_BUF_BASE + USBD_GET_EP_BUF_ADDR(EP2)), (uint8_t *)gRxBuf0, i32Len);
        USBD_SET_PAYLOAD_LEN(EP2, i32Len);
	
    }

    if(comRbytes1 && (gu32TxSize1 == 0))
    {
        i32Len = comRbytes1;
        if(i32Len > EP7_MAX_PKT_SIZE)
            i32Len = EP7_MAX_PKT_SIZE;
    
        for(i=0;i<i32Len;i++)
        {
            gRxBuf1[i] = comRbuf1[comRhead1++];
            if(comRhead1 >= RXBUFSIZE)
                comRhead1 = 0;
        }

        NVIC_DisableIRQ(UART1_IRQn);
        comRbytes1 -= i32Len;
        NVIC_EnableIRQ(UART1_IRQn);

        gu32TxSize1 = i32Len;
        USBD_MemCopy((uint8_t *)(USBD_BUF_BASE + USBD_GET_EP_BUF_ADDR(EP7)), (uint8_t *)gRxBuf1, i32Len);
        USBD_SET_PAYLOAD_LEN(EP7, i32Len);
    }

    /* Process the Bulk out data when bulk out data is ready. */
    if(gi8BulkOutReady0 && (gu32RxSize0 <= TXBUFSIZE - comTbytes0))
    {
        for(i=0;i<gu32RxSize0;i++)
        {
            comTbuf0[comTtail0++] = gpu8RxBuf0[i];
            if(comTtail0 >= TXBUFSIZE)
                comTtail0 = 0;
        }

        NVIC_DisableIRQ(UART0_IRQn);
        comTbytes0 += gu32RxSize0;
        NVIC_EnableIRQ(UART0_IRQn);

        gu32RxSize0 = 0;
        gi8BulkOutReady0 = 0; /* Clear bulk out ready flag */

        /* Ready to get next BULK out */
        USBD_SET_PAYLOAD_LEN(EP3, EP3_MAX_PKT_SIZE);
    }

    if(gi8BulkOutReady1 && (gu32RxSize1 <= TXBUFSIZE - comTbytes1))
    {
        for(i=0;i<gu32RxSize1;i++)
        {
            comTbuf1[comTtail1++] = gpu8RxBuf1[i];
            if(comTtail1 >= TXBUFSIZE)
                comTtail1 = 0;
        }
        
        NVIC_DisableIRQ(UART1_IRQn);
        comTbytes1 += gu32RxSize1;
        NVIC_EnableIRQ(UART1_IRQn);

        gu32RxSize1 = 0;
        gi8BulkOutReady1 = 0; /* Clear bulk out ready flag */

        /* Ready to get next BULK out */
        USBD_SET_PAYLOAD_LEN(EP6, EP6_MAX_PKT_SIZE);
    }

    /* Process the software Tx FIFO */
    if(comTbytes0)
    {
        /* Check if Tx is working */		    
        if((UART0->IER & UART_IER_THRE_IEN_Msk) == 0)
        {
            /* Send one bytes out */
            UART0->DATA = comTbuf0[comThead0++];
            if(comThead0 >= TXBUFSIZE)
                comThead0 = 0;

            NVIC_DisableIRQ(UART0_IRQn);
            comTbytes0--;    
            NVIC_EnableIRQ(UART0_IRQn);
            
            /* Enable Tx Empty Interrupt. (Trigger first one) */  
            UART0->IER |= UART_IER_THRE_IEN_Msk;
        }
    }

    if(comTbytes1)
    {
        /* Check if Tx is working */		    
        if((UART1->IER & UART_IER_THRE_IEN_Msk) == 0)
        {
            /* Send one bytes out */
            UART1->DATA = comTbuf1[comThead1++];
            if(comThead1 >= TXBUFSIZE)
                comThead1 = 0;

            NVIC_DisableIRQ(UART1_IRQn);
            comTbytes1--;    
            NVIC_EnableIRQ(UART1_IRQn);
            
            /* Enable Tx Empty Interrupt. (Trigger first one) */  
            UART1->IER |= UART_IER_THRE_IEN_Msk;
        }
    }
}


/*---------------------------------------------------------------------------------------------------------*/
/*  Main Function									                                           			   */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main (void)
{
    SYS_Init();
    UART0_Init();
    UART1_Init();
   
	/* Initial USB engine */
	USBD_Open(&gsInfo, VCOM_ClassRequest, NULL);

	/* Endpoint configuration */
	VCOM_Init();

	/* Enable USB interrupt */
	NVIC_EnableIRQ(USBD_IRQn);
    NVIC_SetPriority(USBD_IRQn, 0);
    NVIC_SetPriority(UART0_IRQn, 3);
    NVIC_SetPriority(UART1_IRQn, 3);

	/* Start transfer */
    USBD_Start();
	
	GPIO_CLR_PIN(PB14); // Green LED ON

	while(1)
    {
        VCOM_TransferData();
    }
}



/*** (C) COPYRIGHT 2013 Nuvoton Technology Corp. ***/

